"use strict";

var _chai = require("chai");

var _mocha = require("mocha");

var _react = _interopRequireDefault(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _GraphiQL = require("../GraphiQL");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

document.createRange = function () {
  return {
    setEnd: function setEnd() {},
    setStart: function setStart() {},
    getBoundingClientRect: function getBoundingClientRect() {
      return {
        right: 0
      };
    },
    getClientRects: function getClientRects() {
      return {
        right: 0
      };
    }
  };
};

var mockStorage = function () {
  var store = {};
  return {
    getItem: function getItem(key) {
      return store.hasOwnProperty(key) ? store[key] : null;
    },
    setItem: function setItem(key, value) {
      store[key] = value.toString();
    },
    clear: function clear() {
      store = {};
    }
  };
}(); // The smallest possible introspection result that builds a schema.


var simpleIntrospection = {
  data: {
    __schema: {
      queryType: {
        name: 'Q'
      },
      types: [{
        kind: 'OBJECT',
        name: 'Q',
        interfaces: [],
        fields: [{
          name: 'q',
          args: [],
          type: {
            name: 'Q'
          }
        }]
      }]
    }
  }
}; // Spins the promise loop a few times before continuing.

var wait = function wait() {
  return Promise.resolve().then(function () {
    return Promise.resolve();
  }).then(function () {
    return Promise.resolve();
  }).then(function () {
    return Promise.resolve();
  });
};

Object.defineProperty(window, 'localStorage', {
  value: mockStorage
});
(0, _mocha.describe)('GraphiQL', function () {
  var noOpFetcher = function noOpFetcher() {};

  (0, _mocha.it)('should throw error without fetcher', function () {
    (0, _chai.expect)(function () {
      return _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, null));
    }).to.throw('GraphiQL requires a fetcher function');
  });
  (0, _mocha.it)('should construct correctly with fetcher', function () {
    (0, _chai.expect)(function () {
      return _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, {
        fetcher: noOpFetcher
      }));
    }).to.not.throw();
  });
  (0, _mocha.it)('should refetch schema with new fetcher',
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee() {
    var firstCalled, firstFetcher, secondCalled, secondFetcher, instance;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            secondFetcher = function _ref3() {
              secondCalled = true;
              return Promise.resolve(simpleIntrospection);
            };

            firstFetcher = function _ref2() {
              firstCalled = true;
              return Promise.resolve(simpleIntrospection);
            };

            firstCalled = false;
            secondCalled = false;
            // Initial render calls fetcher
            instance = _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, {
              fetcher: firstFetcher
            }));
            (0, _chai.expect)(firstCalled).to.equal(true);
            _context.next = 8;
            return wait();

          case 8:
            // Re-render does not call fetcher again
            firstCalled = false;
            instance.update(_react.default.createElement(_GraphiQL.GraphiQL, {
              fetcher: firstFetcher
            }));
            (0, _chai.expect)(firstCalled).to.equal(false);
            _context.next = 13;
            return wait();

          case 13:
            // Re-render with new fetcher is called.
            instance.update(_react.default.createElement(_GraphiQL.GraphiQL, {
              fetcher: secondFetcher
            }));
            (0, _chai.expect)(secondCalled).to.equal(true);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  })));
  (0, _mocha.it)('should not throw error if schema missing and query provided', function () {
    (0, _chai.expect)(function () {
      return _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, {
        fetcher: noOpFetcher,
        query: "{}"
      }));
    }).to.not.throw();
  });
  (0, _mocha.it)('defaults to the built-in default query', function () {
    var graphiQL = _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, {
      fetcher: noOpFetcher
    }));

    (0, _chai.expect)(graphiQL.getInstance().state.query).to.include('# Welcome to GraphiQL');
  });
  (0, _mocha.it)('accepts a custom default query', function () {
    var graphiQL = _reactTestRenderer.default.create(_react.default.createElement(_GraphiQL.GraphiQL, {
      fetcher: noOpFetcher,
      defaultQuery: "GraphQL Party!!"
    }));

    (0, _chai.expect)(graphiQL.getInstance().state.query).to.equal('GraphQL Party!!');
  });
});